// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_ENVIRONMENT_H
#define OB_ENVIRONMENT_H

namespace CORBA
{

class Environment;
typedef Environment* Environment_ptr;
typedef Environment_ptr EnvironmentRef;

} // End of namespace CORBA

void OBDuplicate(CORBA::Environment_ptr);
void OBRelease(CORBA::Environment_ptr);

namespace CORBA
{

//
// The Environment class
//
typedef OB::ObjVar< Environment > Environment_var;
typedef OB::ObjOut< Environment > Environment_out;

class Environment : public OB::RefCount
{
    //
    // The exception
    //
    Exception* exception_;

    //
    // Hide copy-constructor and assignment operator
    //
    Environment(const Environment&);
    void operator=(const Environment&);

protected:

    Environment();
    virtual ~Environment();

    friend class ORB; // ORBs may create Enviroments
    friend class Request; // Requests may create Enviroments

public:

    //
    // Standard IDL to C++ Mapping
    //
    static inline Environment_ptr _duplicate(Environment_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline Environment_ptr _nil()
    { return 0; }

    void exception(Exception*);
    Exception* exception() const;
    void clear() { exception(0); }
};

inline void
release(Environment_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(Environment_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
